# Import Script
<#
.SYNOPSIS
    This script imports provisioned and deprovisioned users from REST into a Metaverse.

.DESCRIPTION
    The script retrieves user data from REST using the REST API and transforms it into CSEntryChange objects
    that can be imported into the Metaverse. The imported users are then processed and added to the import results.

.PARAMETER ConfigParameters
    The configuration parameters for the script.

.PARAMETER Schema
    The schema of the Metaverse.

.PARAMETER OpenImportConnectionRunStep
    The run step for opening the import connection.

.PARAMETER GetImportEntriesRunStep
    The run step for retrieving the import entries.

.PARAMETER PSCredential
    The credentials for authenticating with the REST API.

.FUNCTIONS
    The script contains the following functions:
    - Write-DebugLog: Writes a debug log message to a log file.
    - GetTokenFromCredentials: Retrieves the token from the provided credentials.
    - Import-RESTProvisionedUsers: Imports provisioned and deprovisioned users from REST.

.OUTPUTS
    The script outputs the import results, which include the CSEntryChange objects to be imported into the Metaverse.

.EXAMPLE
    Import-RESTProvisionedUsers -ConfigParameters $ConfigParameters -Schema $Schema -OpenImportConnectionRunStep $OpenImportConnectionRunStep -GetImportEntriesRunStep $GetImportEntriesRunStep -PSCredential $PSCredential
#>

param(
    [System.Collections.ObjectModel.KeyedCollection[string, Microsoft.MetadirectoryServices.ConfigParameter]]
    [ValidateNotNull()]
    $ConfigParameters,
    [Microsoft.MetadirectoryServices.Schema]
    [ValidateNotNull()]
    $Schema,
    [Microsoft.MetadirectoryServices.OpenImportConnectionRunStep]
    $OpenImportConnectionRunStep,
    [Microsoft.MetadirectoryServices.ImportRunStep]
    $GetImportEntriesRunStep,
    [pscredential]
    $PSCredential
)

#region General Functions

function Write-DebugLog {
    param
    (
        [string]$Message
    )
  
    $logPath = "C:\Logs\ImportScript.log"
    $logMessage = "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") - $Message"
    Add-Content -Path $logPath -Value $logMessage
  
}

function GetTokenFromCredentials {
    param (
      [System.Management.Automation.PSCredential]$Credentials
    )
     Write-DebugLog "Retrieving TOKEN"
    $token = $Credentials.GetNetworkCredential().Password
    Write-DebugLog "TOKEN Retrieval completed"
    $username = $Credentials.GetNetworkCredential().UserName
    Write-DebugLog "USERNAME Retrieval completed"
    $password = $token

  # Create a Base64-encoded string of the credentials
  $auth = $username + ':' + $password
  $Encoded = [System.Text.Encoding]::UTF8.GetBytes($auth)
  $authorizationInfo = [System.Convert]::ToBase64String($Encoded)

  return $authorizationInfo
    
  }

#endregion

#region SCIM Functions

function Import-RESTProvisionedUsers {
    param (
        
    )
    $Token = GetTokenFromCredentials -Credentials $PSCredential

    $headers = @{
        "Accept" = "application/json"
        "Content-Type" = "application/json"
        "Authorization" = "Basic $Token"        
    }
    $results = New-Object System.Collections.Generic.List[PSObject]
    $response = Invoke-RestMethod -Uri 'https://samplerest.faisalrahman.com/Person' -Method 'GET' -Headers $headers
    for ($i = 0; $i -lt $response.count; $i++) {

       $hrefNames = $response.items[$i].links | where { $_.Name -eq "names" } | Select -ExpandProperty href
       $hrefEmails = $response.items[$i].links | where { $_.Name -eq "emails" } | Select -ExpandProperty href
       $hrefWork = $response.items[$i].links | where { $_.Name -eq "workRelationships" } | Select -ExpandProperty href

       $responseNames = Invoke-RestMethod -Uri $hrefNames -Method "GET" -Headers $headers
       $responseEmails = Invoke-RestMethod -Uri $hrefEmails -Method "GET" -Headers $headers
       $responseWork = Invoke-RestMethod -Uri $hrefWork -Method "GET" -Headers $headers
       
       $hrefAssign = $responseWork.items[0].links | where { $_.Name -eq "assignments" } | Select -ExpandProperty href
       $responseAssign = Invoke-RestMethod -Uri $hrefAssign -Method "GET" -Headers $headers
        
        $results.Add([PSCustomObject]@{            
            "LastName" = $responseNames.items.LastName
            "FirstName" = $responseNames.items.FirstName 
            "LegislationCode" = $responseNames.items.LegislationCode
            "EmailType" = $responseEmails.items.EmailType
            "EmailAddress" = $responseEmails.items.EmailAddress
            "PrimaryFlag" = $responseEmails.items.PrimaryFlag
            "LegalEmployerName" = $responseWork.items.LegalEmployerName
            "WorkerType" = $responseWork.items.WorkerType
            "ActionCode" = $responseAssign.items.ActionCode
            "BusinessUnitName" = $responseAssign.items.BusinessUnitName
                            
        })     
       }
       
       Write-DebugLog "Imported Users from REST"
       $responseJSON = $results | ConvertTo-Json
       Write-DebugLog "$responseJSON"
       $results
    }
    
    




#endregion

Set-PSDebug -Strict
Write-DebugLog "Starting Import Script"

$commonModule = (Join-Path -Path ([Microsoft.MetadirectoryServices.MAUtils]::MAFolder) -ChildPath $ConfigParameters['Common Module Script Name (with extension)'].Value)
Import-Module -Name $commonModule -Verbose:$false -ErrorAction Stop

$importResults = New-Object -TypeName 'Microsoft.MetadirectoryServices.GetImportEntriesResults'

$csEntries = New-Object -TypeName 'System.Collections.Generic.List[Microsoft.MetadirectoryServices.CSEntryChange]'

$columnsToImport = $Schema.Types[0].Attributes

Write-DebugLog "Loaded $($columnsToImport.Count) attributes to import"
foreach ($column in $columnsToImport)
{
    Write-DebugLog "Attribute: $($column.Name)"
}

$recordsToImport = Import-RESTProvisionedUsers

Write-DebugLog "Imported $($recordsToImport.Count) records"


foreach ($record in $recordsToImport)
{

    Write-DebugLog 'Starting new record'

    $foundValidColumns = $false

    $entrySchema = $Schema.Types[0];
    $csEntry = New-xADSyncPSConnectorCSEntryChange -ObjectType $entrySchema.Name -ModificationType Add

    foreach ($column in $columnsToImport)
    {

        $columnName = $column.Name

        Write-DebugLog "Processing column $columnName"

        if ($record.$columnName)
        {

            Write-DebugLog 'Found column'

            $foundValidColumns = $true

            $anchorAttrName = $entrySchema.AnchorAttributes[0].Name
            $value = [string]$record.$columnName

            Write-DebugLog "$columnName with value equal $value"


            if ($columnName -eq $anchorAttrName) {


                $csEntry.AnchorAttributes.Add([Microsoft.MetadirectoryServices.AnchorAttribute]::Create($columnName, $value))
            }


            $csEntry | Add-xADSyncPSConnectorCSAttribute -ModificationType Add -Name $columnName -Value ([Collections.IList]($record.$columnName.Split(";")))

        }

    }

    if ($foundValidColumns)
    {

        Write-DebugLog 'Publishing CSEntryChange'

        $csEntries.Add($csEntry)

    }

    Write-DebugLog 'Record completed'

}


$importResults.CSEntries = $csEntries

$importResults.MoreToImport = $false

Write-Output $importResults